﻿//===============================================================================
// PJM Interconnection, LLC. Sprego Application
// Comments provided by Glenn Long
//===============================================================================

using System.Windows.Controls;
using System.ComponentModel;
using PJMBaseLibrary.Utilities.ApplicationEnums;
using PJMBaseLibrary.Utilities.Logger;
using UIComposition.Services;
using UIComposition.Controllers;
using Microsoft.Practices.Unity;
using UIComposition.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using Infragistics.Windows.DataPresenter.Events;

namespace UIComposition.Views.DeselectedResourcesView
{
 
    /// <summary>
    /// Interaction logic for ConstraintsView.xaml
    /// </summary>
    public partial class DeselectedResourcesView : UserControl, IDeselectedResourcesView, IRefreshView
    {
        private int index = 0;
        private static List<string> groupList = new List<string>();

        public DeselectedResourcesView()
        {
            InitializeComponent();
        }

        public DeselectedResourcesPresentationModel Model
        {
            get { return this.DataContext as DeselectedResourcesPresentationModel; }
            set { this.DataContext = value; }           
           
        }

        public void AutoRefreshCurrentView()
        {
            this.DataContext = null;
            IUnityContainer aContainer = ConnectionService.Instance.globalContainer;
            if (aContainer != null)
            {
                IDeselectedResourcesPresenter viewPresenter = aContainer.Resolve<IDeselectedResourcesPresenter>();

                viewPresenter.SetDeselectedResources(Convert.ToDecimal(CasesService.CaseID));
                this.DataContext = viewPresenter.View.Model;
            }
        }


        private void DeselectedResources_AssigningFieldLayoutToItem(object sender, Infragistics.Windows.DataPresenter.Events.AssigningFieldLayoutToItemEventArgs e)
        {

            e.FieldLayout = this.DeselectedResourcesGrid.FieldLayouts[index];
            index++;

            if (index > 0) { index = 0; }           
               

        }


        private void DeselectedResourcesGrid_RecordsInViewChanged(object sender, RecordsInViewChangedEventArgs e)
        {
           string eventName =  e.RoutedEvent.Name.ToString();
           int count = this.DeselectedResourcesGrid.ViewableRecords.Count;
        }

        private void DeselectedResourcesGrid_RecordExpanded(object sender, RecordExpandedEventArgs e)
        {

            //var somethingvisible = this.DeselectedResourcesGrid.ViewableRecords;

            // ok, here we get both group names and the actual records. now we got som'n.
            //var temp = e.Record.DataPresenter.GetRecordsInView(true);       

            // But wait, temp / results only contain the object handles, not the child data. we need to get the actual child records 
            // out of the GroupByRecord items shoved in the temp container
            // Since temp returns all of the Groups we only want stuff from the currently expanded group. 
            // this is represented by the event and what's passed by e.record

            //GroupByRecord stuff = e.Record as GroupByRecord;
            // there are always two ways to skin the cat. 
            // stuff will contain our child collection, but they make you dig for it. not good for us lazy folk

            // do you have any kids?
            if (e.Record.HasChildren == true)
            {
                // kool, this gives me all of the child records under my expanded list - wow!
                var myChildRecords = e.Record.ViewableChildRecords.ToList();

                // ok, what do we do with this now? oh sure, just convert it back into the type we were using before the type
                // was taken over by the grid record type. ok, forget about casting, that won't work. we need to do it manually
                // watch this magic. Infragistics forgot to enter this in the grid for dummies manual.

                // Create an object of my type
                ASEngine.BusinessObjects.Entities.ResourceDeselects myManyCollection = new ASEngine.BusinessObjects.Entities.ResourceDeselects();

                foreach (Infragistics.Windows.DataPresenter.Record rec in myChildRecords)
                {
                    //this is just to verify i am not seeing things
                    Console.Write(((Infragistics.Windows.DataPresenter.DataRecord)(rec)).DataItem.ToString());
                    // Add my converted type to my master collection
                    myManyCollection.Add((ASEngine.BusinessObjects.Entities.ResourceDeselect)((Infragistics.Windows.DataPresenter.DataRecord)(rec)).DataItem);
                    //DeselectedResourcesHelper.ExpandedResourcesList.Add((ASEngine.BusinessObjects.Entities.ResourceDeselect)((Infragistics.Windows.DataPresenter.DataRecord)(rec)).DataItem);
                   
                }

                // Set the collection to null first, here comes a new one - no don't do this it forces the timer to take over. just overwrite existing
                //DeselectedResourcesHelper.ExpandedResourcesList.Clear();
            
                ListExpandedDeselectedResourcesList refreshExpandedList = new ListExpandedDeselectedResourcesList(myManyCollection);

            }   

                   
        }           
       

       }

    public class DeselectedResourcesHelper : INotifyPropertyChanged
    {
        public int Index { get; set; }
        public string Source = null;
        //public DeselectedResourcesView currentView = new DeselectedResourcesView();

        public DeselectedResourcesHelper(ASEngine.BusinessObjects.Entities.ResourceDeselects units, string source)
        {
            if (source == "ParentSource")
            {
                _units = units;
            }
            else
            {   
                // Store the group of records to be updated if the user groups the items
                // We need to get the unique id's of each record and update them against the master.
                _expunits = units;
            }



            if (UnitResourcesList != null)
            {
                foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect unit in UnitResourcesList)
                    unit.PropertyChanged += (sender, e) =>
                    {
                        // Save Modified
                        ASEngine.BusinessObjects.Entities.ResourceDeselect o = (ASEngine.BusinessObjects.Entities.ResourceDeselect)sender;
                        o.IsDirty = true;
                        //send update.
                        UpdateBO(o);

                        // Raising PropertyChanged for the AllUnitsAreChecked
                        // property causes the data binding system to query its
                        // getter for the new value.
                        if (e.PropertyName == "Tier1Deselected")
                        {
                            this.OnPropertyChanged("AllT1UnitsAreChecked");
                        }
                        else if (e.PropertyName == "Tier2Deselected")
                        {
                            this.OnPropertyChanged("AllT2UnitsAreChecked");
                        }
                        else if (e.PropertyName == "RegDeselected")
                        {
                            this.OnPropertyChanged("AllRegDeSelectAreChecked");
                        }

                    };      

            }



            this.Index = 0;


        }

        ASEngine.BusinessObjects.Entities.ResourceDeselects _units;

        public ASEngine.BusinessObjects.Entities.ResourceDeselects UnitResourcesList { get { return _units; } }

        public static ASEngine.BusinessObjects.Entities.ResourceDeselects _expunits;

        public static ASEngine.BusinessObjects.Entities.ResourceDeselects ExpandedResourcesList { get { return _expunits; } }
        



        public decimal? AllT1UnitsAreChecked
        {
            get
            {
                try
                {
                    // Determine if all units have the same 
                    // value for the IsChecked property.
                    decimal? value = null;

                   for (int i = 0; i < this.UnitResourcesList.Count; ++i)
                        {
                            if (i == 0)
                            {
                                value = _units[0].Tier1deselected;
                            }
                            else if (value != _units[i].Tier1deselected)
                            {
                                value = null;
                                break;
                            }
                        } 
                   

                    return value;

                }
                catch (Exception ex) { Logger.LogException(LogSourceType.ServerApplication, ex); }

                return null;
            }
            set
            {
                if (value == null)
                {
                    return;
                }
                else
                {
                    if (DeselectedResourcesHelper.ExpandedResourcesList != null)
                    {
                        // Update the master list of units based on the expanded selection.

                        foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect expunit in DeselectedResourcesHelper.ExpandedResourcesList)
                        {
                            expunit.Tier1deselected = value.Value;

                            if (this.UnitResourcesList.Contains(expunit))
                            {
                                this.UnitResourcesList.Remove(expunit);
                                this.UnitResourcesList.Add(expunit);
                            }                  

                        }
                    }
                    else 
                    {
                        foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect unit in this.UnitResourcesList)
                        {
                            unit.Tier1deselected = value.Value;
                        }
                    }
                }
            }
        }

        void unit_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == null)
            {
                Console.Write("Property Not Defined");
            }
        }




        public decimal? AllT2UnitsAreChecked
        {
            get
            {
                try
                {
                   // Determine if all units have the same 
                   // value for the IsChecked property.
                   decimal? value = null;

                   for (int i = 0; i < this.UnitResourcesList.Count; ++i)
                        {
                            if (i == 0)
                            {
                                value = _units[0].Tier2deselected;
                            }
                            else if (value != _units[i].Tier2deselected)
                            {
                                value = null;
                                break;
                            }
                        }                    

                    return value;
                }
                catch (Exception ex) { Logger.LogException(LogSourceType.ServerApplication, ex); }

                return null;
            }
            set
            {
                if (value == null)
                {
                    return;
                }
                else
                {
                    if (ExpandedResourcesList != null)
                    {
                        foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect expunit in DeselectedResourcesHelper.ExpandedResourcesList)
                        {
                            expunit.Tier2deselected = value.Value;

                            if (this.UnitResourcesList.Contains(expunit))
                            {
                                this.UnitResourcesList.Remove(expunit);
                                this.UnitResourcesList.Add(expunit);
                            }  
                        }
                    }
                    else
                    {
                        foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect unit in this.UnitResourcesList)
                        {
                            unit.Tier2deselected = value.Value;
                        }
                    }
                }
            }
        }

        // AllRegDeSelectAreChecked

        public decimal? AllRegDeSelectAreChecked
        {
            get
            {
                try
                {
                    // Determine if all units have the same 
                    // value for the IsChecked property.
                    decimal? value = null;
                    for (int i = 0; i < this.UnitResourcesList.Count; ++i)
                        {
                            if (i == 0)
                            {
                                value = _units[0].RegDeselected;
                            }
                            else if (value != _units[i].RegDeselected)
                            {
                                value = null;
                                break;
                            }
                        }
                    

                    return value;
                }
                catch (Exception ex) { Logger.LogException(LogSourceType.ServerApplication, ex); }

                return null;
            }
            set
            {
                if (value == null)
                {
                    return;
                }
                else
                {
                    if (ExpandedResourcesList != null)
                    {
                        foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect expunit in DeselectedResourcesHelper.ExpandedResourcesList)
                        {
                            expunit.RegDeselected = value.Value;

                            if (this.UnitResourcesList.Contains(expunit))
                            {
                                this.UnitResourcesList.Remove(expunit);
                                this.UnitResourcesList.Add(expunit);
                            }  
                        }
                    }
                    else
                    {
                        foreach (ASEngine.BusinessObjects.Entities.ResourceDeselect unit in this.UnitResourcesList)
                        {
                            unit.RegDeselected = value.Value;
                        }
                    }

                }
            }
        }

        

        public void UpdateBO(ASEngine.BusinessObjects.Entities.ResourceDeselect item)
        {
            SaveController.Instance.AddUpdate(typeof(ASEngine.BusinessObjects.Entities.ResourceDeselects), item);            
        }
 

        

        #region INotifyPropertyChanged Members     
      

        public event PropertyChangedEventHandler PropertyChanged;

        void OnPropertyChanged(string prop)
        {           
            if (this.PropertyChanged == null)
            {
                this.PropertyChanged += new PropertyChangedEventHandler(PropertyChanged);
            } 
            else
            {
                 this.PropertyChanged(this, new PropertyChangedEventArgs(prop));               
            }
        }

        #endregion // INotifyPropertyChanged Members


    }

    public class ListDeselectedResourcesList : List<DeselectedResourcesHelper>
    {

        public ListDeselectedResourcesList(decimal caseId)
        {
            DeselectedResourcesHelper resourcesHelper;

            ASEngine.BusinessObjects.Entities.ResourceDeselects resources = DeselectedResourcesService.cache.TryGetByKey(caseId);
            Add(resourcesHelper = new DeselectedResourcesHelper(resources, "ParentSource"));

        }
    }

    public class ListExpandedDeselectedResourcesList : List<DeselectedResourcesHelper>
    {

        public ListExpandedDeselectedResourcesList(ASEngine.BusinessObjects.Entities.ResourceDeselects resources)
        {
            DeselectedResourcesHelper resourcesHelper;

            // Just add the Collection as it will be new each time
            Add(resourcesHelper = new DeselectedResourcesHelper(resources, "ExpandedSource"));

        }
    }



    
}